import mitt from "mitt";

export interface Editor {
  username: string;
  firstname: string;
  lastname: string;
  color: string;
}

export interface EditorState {
  field: string;
  cursor: any;
  range: any;
}

export type AwarenessActions = "update" | "remove";

export type AwarenessEvents = {
  update: { data: EditorState };
  change: { socketId: string; action: AwarenessActions } & EditorState;
};

export class Awareness {
  public readonly editors = new Map<string, Editor>();
  public readonly editorStates = new Map<string, EditorState>();
  public readonly emitter = mitt<AwarenessEvents>();

  public getEditors() {
    return this.editors;
  }

  public getEditor(key: string) {
    return this.editors.get(key);
  }

  public getEditorsByField(field: string) {
    return Array.from(this.editorStates.entries())
      .filter(([key, val]) => val.field == field)
      .map(([key, val]) => key);
  }

  public getEditorObjsByField(field: string) {
    return Array.from(this.editors.entries())
      .filter(([key, val]) => this.getEditorsByField(field).includes(key))
      .map(([key, val]) => val);
  }

  public getEditorStates() {
    return this.editorStates;
  }

  public updateEditor(socketId: string, { action, data }: { action: AwarenessActions; data: Editor }) {
    if (action == "update") {
      this.editors.set(socketId, data);
    } else if (action == "remove") {
      this.editors.delete(socketId);
    }
  }

  public updateEditorState(socketId: string, { action, data }: { action: AwarenessActions; data: EditorState }) {
    if (action == "update") {
      this.editorStates.set(socketId, data);
    } else if (action == "remove") {
      this.editorStates.delete(socketId);
    }
    this.emitter.emit("change", { socketId, action, ...data });
  }

  public publishMyState(data: EditorState) {
    this.emitter.emit("update", { data });
  }

  public reset() {
    this.editors.clear();
    this.editorStates.clear();
  }

  public destroy() {
    this.emitter.all.clear();
    this.reset();
  }
}
